library flutter_appcenter;

import 'dart:async';
import 'dart:io';

import 'package:android_metadata/android_metadata.dart';
import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_appcenter/comment/check_permission.dart';
import 'package:flutter_appcenter/comment/configs.dart';
import 'package:flutter_appcenter/comment/dao.dart';
import 'package:flutter_appcenter/comment/install_app.dart';
import 'package:flutter_appcenter/components/update_dialog.dart';
import 'package:open_appstore/open_appstore.dart';
import 'package:package_info/package_info.dart';
import 'package:progress_dialog/progress_dialog.dart';
import 'package:url_launcher/url_launcher.dart';

class FlutterAppCenter {
  static const MethodChannel _channel = const MethodChannel('flutter_appcenter');
  static String appSecret = '';
  static String token = '';
  static String appId = '';
  static String betaUrl = '';

  static var dialogUpdateSingle;

  /// start running the AppCenert
  static Future<String> init({
    @required String appSecretAndroid,
    @required String appSecretIOS,
    String tokenAndroid,
    String tokenIOS,
    String appIdAndroid,
    String appIdIOS,
    String betaUrlAndroid,
    String betaUrlIOS,

    /// set private or public distribution group
    /// defualut value is publice
    bool usePrivateTrack = false,

    /// when automaticCheckForUpdate is false, can use checkForUpdate method to update
    // bool automaticCheckForUpdate = true,
  
  }) async {
    Map<String, Object> args = {
      'appSecret': Platform.isAndroid ? appSecretAndroid : appSecretIOS,
      'usePrivateTrack': usePrivateTrack,
      /// if true, check update for showing dialog
      'automaticCheckForUpdate': false,
    };

    appSecret = args['appSecret'];
    if(Platform.isAndroid){
      token = tokenAndroid ?? '';
      appId = appIdAndroid ?? '';
      betaUrl = betaUrlAndroid?? '';
    }else{
      token = tokenIOS ?? '';
      appId = appIdIOS ?? '';
      betaUrl = betaUrlIOS ?? '';
    }

    try {
      return await _channel.invokeMethod('initAppCenter', args);
    } on PlatformException catch (e) {
      print("Failed to start running the AppCenert: '${e.message}'.");
      return '$e';
    }
  }

  /// Check if App Center Distribute is enabled
  static Future<bool> isEnabledForDistribute() async{
    try {
      String result = await _channel.invokeMethod('isEnabledForDistribute');
      return result == '1';
    } on PlatformException catch (e) {
      print("Failed to check for Distribute: '${e.message}'.");
      return false;
    }
  }

  /// update dialog
  static Future<bool> checkForUpdate(BuildContext context,{
    downloadUrlAndroid: '',
    Map<String, String> dialog
  }) async{
    final ProgressDialog _progress = ProgressDialog(context,isDismissible: false);
    PackageInfo _packageInfo = await PackageInfo.fromPlatform();
    var _requestResult;

    if(dialog == null){
      dialog = {
        'title': 'App update avaiable',
        'subTitle': 'The lastest version',
        'content': '',
        'confirm': 'Confirm',
        'cancel': 'Postpone',
        'downloading': 'Downloading File...'
      };
    }else{
      dialog['title'] = dialog.containsKey('title') ? dialog['title'] : 'App update avaiable';
      dialog['subTitle'] = dialog.containsKey('subTitle') ? dialog['subTitle'] : '';
      dialog['content'] = dialog.containsKey('content') ? dialog['content'] : ''; // defualt to show the releaseNotes and version
      dialog['confirm'] = dialog.containsKey('confirm') ? dialog['confirm'] : 'Confirm';
      dialog['cancel'] = dialog.containsKey('cancel') ? dialog['cancel'] : 'Postpone';
      dialog['downloading'] = dialog.containsKey('downloading') ? dialog['downloading'] : 'Downloading File...';
    }

    if(token == null || token.isEmpty){
      throw('token must be not null.');
    }

    void checkUpdateApp() async{
     await _progress.show();
      if(Platform.isAndroid){
        Map<String, dynamic> metadata = await AndroidMetadata.metaDataAsMap;
        if(metadata['UMENG_CHANNEL'] == 'pr'){
          await _progress.hide();
          try{
            OpenAppstore.launch(androidAppId: _packageInfo.packageName);
          }catch(e){
            String storeLink = Configs.googlePlayStore(_packageInfo.packageName);

            if (await canLaunch(storeLink)) {
              await launch(storeLink);
            } else {
              throw('can not open google play');
            }
          }
        }else{
          if(await checkPermission()){
            if(downloadUrlAndroid.isNotEmpty){
              _requestResult['download_url'] = downloadUrlAndroid;
            }
            print(_requestResult['download_url']);
            try{
              await installApk(_requestResult['download_url'],_packageInfo.packageName,(received, total) async{
                _progress.update(
                  message: dialog['downloading'],
                  progress: received/total,
                  progressWidget: Container(
                    padding: EdgeInsets.all(8.0), child: CircularProgressIndicator(
                      value: received/total
                    )
                  ),
                  maxProgress: 1,
                );

                if(received == total){
                  await _progress.hide();
                }
              });
            }catch(e){
              _progress.update(
                message: 'fail to update: $e'
              );
              await _progress.hide();
            }
          }
        }
      }else{
        await _progress.hide();

        if(appId.isNotEmpty){
          OpenAppstore.launch(iOSAppId: appId);
        }else{
          if (await canLaunch(betaUrl)) {
            await launch(betaUrl);
          } else {
            if(betaUrl.contains('itms-beta')){
              await launch(betaUrl.replaceFirst(new RegExp(r'itms-beta'), 'https'));
            }else{
              print('can not open testflight');
            }
          }
        }
      }
    }

    void showUpdateDialog() async{
      String content = '';
      if(dialog['content'].isNotEmpty){
        content = dialog['content'];
      }else{
        content = dialog['subTitle'] + ': ${_requestResult['short_version']}(${_requestResult['version']})\n${_requestResult['release_notes']}';
      }

      if(dialogUpdateSingle == null){
        dialogUpdateSingle = await showDialog(
          context: context,
          builder: (BuildContext ctx) {
            return UpdateDialog(
              mandatoryUpdate: _requestResult['mandatory_update'],
              title: '${dialog['title']}',
              content: content,
              confirm: dialog['confirm'],
              cancel: dialog['cancel'],
              onConfirm: () => checkUpdateApp(),
            );
          }
        );
      }else{
        dialogUpdateSingle();
      }
    }

    /// step 1: abtain the app vesion
    String _version = _packageInfo.version;
    String _buildNumber = _packageInfo.buildNumber;
    print('local version: $_version+$_buildNumber');
    if(_version == _buildNumber){
      _buildNumber = '0';
    }

    /// step 2: require latest version from AppCenter
    Dao.token = token;
    var dao = new Dao();
    _requestResult = await dao.get(url: Configs.latestVersionUrl(appSecret));

    print('latestUrl:${Configs.latestVersionUrl(appSecret)}');
    print('remote version: ${_requestResult['short_version']}+${_requestResult['version']}');

    /// step 3: compare the app's verion and build number with the remote's
    if(_version != _requestResult['short_version']){
      List<String> versionArr = _version.split('.');
      List<String> remoteVersionArr = _requestResult['short_version'].split('.');

      for(int i = 0; i < remoteVersionArr.length; i++){
        if(int.parse(remoteVersionArr[i]) > int.parse(versionArr[i])){
          showUpdateDialog();
          return true;
        }
      }
    }else if(_buildNumber != _requestResult['version'] && int.parse(_requestResult['version']) > int.parse(_buildNumber)){
      showUpdateDialog();
      return true;
    }

    return false;
  }
}
